# -*- python -*-
# $RCSfile: destinationparamwidget.py,v $
# $Revision: 1.13 $
# $Author: langer $
# $Date: 2006/03/09 20:22:20 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# Widget for the Destination parameter.  Needs a separate file because
# it uses the fileselector, which in turn depends on stuff in
# parameterwidgets.py, which otherwise would be the obvious place for
# this.

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common.IO import automatic
from oof2.common.IO import destinationparam
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import fileselector
from oof2.common.IO.GUI import parameterwidgets
import gtk

MSGWINDOW = "Message Window"

class DestinationParamWidget(parameterwidgets.ParameterWidget):
    # All instances share the same namelist.
    namelist = [MSGWINDOW]
    def __init__(self, param=None, scope=None):
        parameterwidgets.ParameterWidget.__init__(self, gtk.Frame(), scope)
        vbox = gtk.VBox(spacing=2)
        self.gtk.add(vbox)

        self.chooser = chooser.ChooserWidget(DestinationParamWidget.namelist,
                                             callback=self.chooserCB)
        vbox.pack_start(self.chooser.gtk, expand=0, fill=0)

        bbox = gtk.HBox(spacing=2, homogeneous=True)
        vbox.pack_start(bbox, expand=0, fill=0)

        newbutton = gtk.Button("New...")
        bbox.pack_start(newbutton, expand=1, fill=1)
        newbutton.connect('clicked', self.newCB)

        self.clearbutton = gtk.Button("Clear")
        bbox.pack_start(self.clearbutton, expand=1, fill=1)
        self.clearbutton.connect('clicked', self.clearCB)

        switchboard.requestCallback('destinations changed', self.rebuild)
        self.rebuild()
        self.widgetChanged(1, interactive=0)

    def rebuild(self):
        self.chooser.update(DestinationParamWidget.namelist)

    def newCB(self, button):
        newname = fileselector.fileSelector(ident="destwidget", mode="w",
                                            title="Data destination")
        if newname and newname not in DestinationParamWidget.namelist:
            DestinationParamWidget.namelist.append(newname)
            self.rebuild()
            self.just_set_value(newname)
            self.widgetChanged(1, interactive=1)
    def clearCB(self, button):
        DestinationParamWidget.namelist = [MSGWINDOW]
        self.rebuild()
        self.widgetChanged(1, interactive=1)
    def chooserCB(self, *args):
        self.widgetChanged(1, interactive=1)
    def just_set_value(self, value):
        if value == automatic.automatic:
            self.chooser.set_state(MSGWINDOW)
        else:
            self.chooser.set_state(value)
    def set_value(self, value):
        self.just_set_value(value)
        self.widgetChanged(1, interactive=0)        
    def get_value(self):
        val = self.chooser.get_value()
        if val == MSGWINDOW:
            return automatic.automatic
        return val
        

##        # Locally handle the GtkOptionMenu, so we can do separators.
##        gtkobj = gui_utils.SizedOptionMenu()
##        self.menu = GtkMenu()
##        gtkobj.set_menu(self.menu)

##        self.newfile_item = GtkMenuItem("New...")
##        self.menu.add(self.newfile_item)
##        self.newfile_item.connect("activate", self.item_activate,
##                                  "New...")
##        guitop.top().tooltip(self.newfile_item,
##                             "Select a file to receive data.")

##        self.clearfile_item = GtkMenuItem("Clear")
##        self.menu.add(self.clearfile_item)
##        self.clearfile_item.connect("activate", self.item_activate,
##                                    "Clear")
##        guitop.top().tooltip(self.clearfile_item,
##                             "Clear the list of data files.")

##        separator = GtkMenuItem()
##        self.menu.add(separator)

##        self.auto_item = GtkMenuItem("Message Window")
##        self.menu.add(self.auto_item)
##        self.auto_item.connect("activate", self.item_activate,
##                               "Message Window")
##        guitop.top().tooltip(self.auto_item, "Send data to the Message Window")

##        separator = GtkMenuItem()
##        self.menu.add(separator)
        
##        self.item_callbacks = {}

##        # Switchboard callback tells the other instances when
##        # the list has changed.
##        switchboard.requestCallback("destinations changed",
##                                    self.rebuild)
##        parameterwidgets.ParameterWidget.__init__(self, gtkobj, scope)

##        if param:
##            self.value = param.value
##        else:
##            self.value = automatic.automatic

##        self.rebuild()
##        self.widgetChanged(1, interactive=0)
        

##    def rebuild(self):
##        for (item, callback_id) in self.item_callbacks.items():
##            item.disconnect(callback_id)
##            self.menu.remove(item)
##        self.item_callbacks = {}
##        for n in DestinationParamWidget.namelist:
##            new_item = GtkMenuItem(n)
##            self.menu.add(new_item)
##            cb = new_item.connect("activate", self.item_activate, n)
##            self.item_callbacks[new_item] = cb
##        self.just_set_value(self.value)
            
##    def item_activate(self, gtkobj, name):
##        if name=="Message Window":
##            self.value = automatic.automatic
##        elif name=="Clear":
##            DestinationParamWidget.namelist = []
##            switchboard.notify("destinations changed")
##        elif name=="New...":
##            new_name = fileselector.fileSelector(ident="destwidget")
##            if new_name:
##                try:
##                    idx = self.namelist.index(new_name)
##                except ValueError:
##                    self.namelist.append(new_name)
##                    new_item = GtkMenuItem(new_name)
##                    self.menu.add(new_item)
##                    cb = new_item.connect("activate",
##                                          self.item_activate, new_name)
##                    self.item_callbacks[new_item] = cb
##                    switchboard.notify("destinations changed")
##                self.just_set_value(new_name)
##            else:
##                # Null result from fileselector, reset to old state.
##                self.just_set_value(self.value)
##        else:
##            self.just_set_value(name)
##        self.widgetChanged(1, interactive=1)

##    def set_value(self, value):
##        self.just_set_value(value)
##        self.widgetChanged(1, interactive=0)
    
##    def just_set_value(self, value):
##        debug.mainthreadTest()
##        if value==automatic.automatic:
##            self.gtk.set_history(3)
##        else:
##            try:
##                idx = DestinationParamWidget.namelist.index(value)
##            except ValueError:
##                self.gtk.set_history(3)
##                value = automatic.automatic
##            else:
##                self.gtk.set_history(idx+5)
##        self.value = value
##        self.gtk.show_all()
            
##    def get_value(self):
##        return self.value


def _make_DestParamWidget(self, scope=None):
    return DestinationParamWidget(self, scope)

destinationparam.DestinationParameter.makeWidget = _make_DestParamWidget
