// -*- C++ -*-
// $RCSfile: mpitools.C,v $
// $Revision: 1.23 $
// $Author: redwing $
// $Date: 2004/07/20 20:15:50 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include <string>
#include <Python.h>
#include <iostream>
#include <mpi.h>
#include "mpitools.h"
#include "ooferror.h"
#include "tostring.h"
#include "trace.h"


MPIException::MPIException(const std::string &m, const std::string &f, int l, int proc_num):  msg(m), file(f), line(l), process(proc_num) {
  
}



std::string MPIException::pythonequiv() const {
  return  "process number: " + to_string(process) + "\n" + " MPIException('" 
    + msg + "', '" + file + "', " + to_string(line) + ")";

}


void _mpi_initialize(std::vector<char*> *argh) {
  int argc = argh->size();
  char **argv;
  if (argc == 1)
    {
      argc = 1;
      argv = (char **)malloc(argc * sizeof(char *));
      argv[0] = "           ";
    }
  else
    {
      argv = (char **)malloc(argc * sizeof(char *));
    
      for(int i=0; i<argh->size(); i++)
	{
	  
	  argv[i] = (*argh)[i];
	}
    }
  int state, result;
  //char processor_name[MPI_MAX_PROCESSOR_NAME];
  state = MPI_Init(&argc, &argv); //initializes MPI
  //state = MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &result); //initializes MPI
  //std::cerr << "state=" << state << std::endl;
  //std::cerr << "result=" << result << std::endl;






 }

void mpi_finalize() {
  MPI_Finalize();
  //  std::cerr << "Quiting MPI" << std::endl;
}

int mpi_initialized() {
  int error, flag;
  error = MPI_Initialized(&flag);
  return flag;

}

int size()
{
  int local_size;
  MPI_Comm_size(getOOF_Communicator().get(),&local_size);
  return local_size;
}

int rank(){
  int myrank;
  MPI_Comm_rank(getOOF_Communicator().get(),&myrank);
  return myrank;
}


std::string get_processor_name() {
  char processor_name[MPI_MAX_PROCESSOR_NAME];
  int  namelen;

  MPI_Get_processor_name(processor_name,&namelen);
  return processor_name;
}


int mpi_abort() {
  int code=0;
  return MPI_Abort(getOOF_Communicator().get(), code);
}


int mpi_barrier() {
  return MPI_Barrier(getOOF_Communicator().get()/*MPI_COMM_WORLD*/);
}



//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\



int send_string(const std::string & message, int destination) {
  /* 
     Message is sent in two steps.
     First send size of message and then send actual message.
  */
  int tag = 0;
  int length = std::strlen(message.c_str())+1;
  int * p_length = &length;
  // std::cerr << "before MPI_Send 1" << std::endl;
  int size_status = MPI_Send(p_length, 1, MPI_INT, destination, tag, getOOF_Communicator().get());
  tag = 1;
  //std::cerr << "before MPI_Send 2" << std::endl;
  int message_status = MPI_Send((char *) message.c_str(), length, MPI_CHAR, destination, tag, getOOF_Communicator().get());
  return message_status && size_status;
}



std::string receive_string(int source) {
  /*
    Message is received in two steps.
    First, receive size of message and then allocate enough memory to catch it. 
   */
  int tag = 0;
  
  int err, st_length, int_length;
  int * p_length = new int;
  MPI_Status int_status, status;
  

  // get size of message 
  // std::cerr << "before MPI_Recv 1" << std::endl;
  int size_status = MPI_Recv(p_length, 1, MPI_INT, source, tag, getOOF_Communicator().get(), &int_status);
  MPI_Get_count(&int_status, MPI_INT, &int_length); 
  int length = *(p_length);
  //std::cerr << length << std::endl;
  // allocate memory for actual message and grab it
  tag = 1;
  length += 1;
  char *temp = new char [length];
  //std::cerr << "before MPI_Recv 2" << std::endl;
  err = MPI_Recv(temp, length, MPI_CHAR, source, tag, getOOF_Communicator().get(), &status);
   
  MPI_Get_count(&status, MPI_CHAR, &st_length); 
  // status.st_length is not available in all MPI implementations
  //Alternative is: MPI_Get_elements(MPI_Status *, MPI_Datatype, int *);
  std::string message = temp;
  delete p_length;
  return message;  

} 

int send_int(int & message, int destination) {
  int tag = 0;
  int * p_length = &message;
  int message_status = MPI_Send(p_length, 1, MPI_INT, destination, tag, getOOF_Communicator().get());
  return message_status;
}

int receive_int(int source) {
  int tag = 0;
  int int_length;
  int * p_length = new int;
  MPI_Status int_status;
  // get size of message 
  // std::cerr << "before MPI_Recv 1" << std::endl;
  int size_status = MPI_Recv(p_length, 1, MPI_INT, source, tag, getOOF_Communicator().get(), &int_status);
  MPI_Get_count(&int_status, MPI_INT, &int_length); 
  int length = *(p_length);
  delete p_length;
  return length;

} 



int send_vector_of_ints(std::vector<int> * message, int destination)
{
  int tag = 0;
  int length = message->size();
  int * p_length = &length;
  // std::cerr << "before MPI_Send 1" << std::endl;
  int size_status = MPI_Send(p_length, 1, MPI_INT, destination, tag, getOOF_Communicator().get());
  tag = 1;
  int message_status = 0;
  if (length>0)
    message_status  = MPI_Send(&(*message)[0], length, MPI_INT, destination, tag, getOOF_Communicator().get());
  return message_status && size_status;
 
 

}

std::vector<int>  * receive_vector_of_ints(int origin)
{
  int tag = 0;
  
  int err, st_length, int_length;
  int * p_length = new int;
  MPI_Status int_status, status;
  

  // get size of message 
  // std::cerr << "before MPI_Recv 1" << std::endl;
  int size_status = MPI_Recv(p_length, 1, MPI_INT, origin, tag, getOOF_Communicator().get(), &int_status);
  MPI_Get_count(&int_status, MPI_INT, &int_length); 
  int length = *(p_length);
  //std::cerr << length << std::endl;
  // allocate memory for actual message and grab it
  tag = 1;
  std::vector<int> * temp = new std::vector<int>(length);
  if (length>0)
    {
      err = MPI_Recv(&(*temp)[0], length, MPI_INT, origin, tag, getOOF_Communicator().get(), &status);
      MPI_Get_count(&status, MPI_INT, &st_length); 
    }
  delete p_length;
  // status.st_length is not available in all MPI implementations
  //Alternative is: MPI_Get_elements(MPI_Status *, MPI_Datatype, int *);
  return temp;

}



int send_vector_of_doubles(std::vector<double> * message, int destination)
{
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int tag = 0;
  int length = message->size();
  int * p_length = &length;
  // std::cerr << "before MPI_Send 1" << std::endl;
  int size_status = MPI_Send(p_length, 1, MPI_INT, destination, tag, getOOF_Communicator().get());
  tag = 1;
  //std::cerr << "before MPI_Send 2" << std::endl;
  int message_status = 0;
  if (length>0)
    message_status = MPI_Send(&(*message)[0], length, MPI_DOUBLE, destination, tag, getOOF_Communicator().get());
  return message_status && size_status;

}

std::vector<double> *  receive_vector_of_doubles(int origin)
{
  /*
    Message is received in two steps.
    First, receive size of message and then allocate enough memory to catch it. 
   */
  int tag = 0;
  
  int err, st_length, int_length;
  int * p_length = new int;
  MPI_Status int_status, status;
  

  // get size of message 
  // std::cerr << "before MPI_Recv 1" << std::endl;
  int size_status = MPI_Recv(p_length, 1, MPI_INT, origin, tag, getOOF_Communicator().get(), &int_status);
  MPI_Get_count(&int_status, MPI_INT, &int_length); 
  int length = *(p_length);
  //std::cerr << length << std::endl;
  // allocate memory for actual message and grab it
  tag = 1;
  std::vector<double> * temp = new std::vector<double>(length);
  //std::cerr << "before MPI_Recv 2" << std::endl;
  if (length>0)
    {
      err = MPI_Recv(&(*temp)[0], length, MPI_DOUBLE, origin, tag, getOOF_Communicator().get(), &status);
      
      MPI_Get_count(&status, MPI_INT, &st_length); 
    }
  // status.st_length is not available in all MPI implementations
  //Alternative is: MPI_Get_elements(MPI_Status *, MPI_Datatype, int *);
  delete p_length;
  return temp;

}





//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\


int broadcast_string(const std::string & message, int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int length = std::strlen(message.c_str())+1;
  int * p_length = &length;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  int message_status = MPI_Bcast((char *) message.c_str(), length, MPI_CHAR, origin, getOOF_Communicator().get());
  //std::cerr << "status of sent message" << message_status  << size_status << std::endl;
  return message_status && size_status;
}


std::string receive_broadcast_string(int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int * p_length = new int;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  int length = *(p_length);
  length += 1;
  char *temp = new char [length];
  int message_status = MPI_Bcast(temp, length, MPI_CHAR, origin, getOOF_Communicator().get());
  //std::cerr << "status of recieved message" << message_status << size_status << std::endl;
  // std::string message = temp;
  delete p_length;
  return temp;
}


int broadcast_int(int & message, int origin) {

  int * p_message = &message;
  //std::cerr << "inside broadcast. before MPI_Bcast send" << std::endl;
  int state  = MPI_Bcast(p_message, 1, MPI_INT, origin, getOOF_Communicator().get());
  //  mpi_barrier();
  //std::cerr << "inside broadcast. after MPI_Bcast send" << std::endl;
  //delete p_message;
  return state;
}


int receive_broadcast_int(int origin) {
  int  msg = 0;
  //std::cerr << "inside broadcast. before MPI_Bcast receive" << std::endl;
  MPI_Bcast(&msg, 1, MPI_INT, origin, getOOF_Communicator().get());
  //std::cerr << "inside broadcast. after MPI_Bcast receive" << std::endl;
  return msg;
}

int broadcast_double (double & message, int origin) {
  return MPI_Bcast(&message, 1, MPI_DOUBLE, origin, getOOF_Communicator().get());

}


double receive_broadcast_double(int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  double  msg = 0;
  MPI_Bcast(&msg, 1, MPI_DOUBLE, origin, getOOF_Communicator().get());
  return msg;
}

int broadcast_vector_of_shorts(std::vector<unsigned short> * message, int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int length = message->size();
  int * p_length = &length;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  //unsigned short * list = message[0];
  int message_status = MPI_Bcast(&(*message)[0], length, MPI_UNSIGNED_SHORT, origin, getOOF_Communicator().get());
  // int message_status = MPI_Bcast((unsigned short *) list, length, MPI_UNSIGNED_SHORT, origin, getOOF_Communicator().get());
  //std::cerr << "status of sent message" << message_status  << size_status << std::endl;
  return message_status && size_status;
}


std::vector<unsigned short> *  receive_broadcast_vector_of_shorts(int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int * p_length = new int;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  int length = *(p_length);
  std::vector<unsigned short> * temp = new std::vector<unsigned short>(length);
  int message_status = MPI_Bcast(&(*temp)[0], length, MPI_UNSIGNED_SHORT, origin, getOOF_Communicator().get());
  //std::cerr << "status of recieved message" << message_status << size_status << std::endl;
  // std::string message = temp;
  delete p_length;
  return temp;
}


int broadcast_vector_of_ints(std::vector<int> * message, int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int length = message->size();
  int * p_length = &length;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  //unsigned short * list = message[0];
  
  int message_status = 0;
  if (length>0)
    message_status = MPI_Bcast(&(*message)[0], length, MPI_INT, origin, getOOF_Communicator().get());
  // int message_status = MPI_Bcast((unsigned short *) list, length, MPI_UNSIGNED_SHORT, origin, getOOF_Communicator().get());
  //std::cerr << "status of sent message" << message_status  << size_status << std::endl;
  return message_status && size_status;
}


std::vector<int> *  receive_broadcast_vector_of_ints(int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int * p_length = new int;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  int length = *(p_length);
  std::vector<int> * temp = new std::vector<int>(length);

  int message_status = 0;
  if (length>0)
    message_status = MPI_Bcast(&(*temp)[0], length, MPI_INT, origin, getOOF_Communicator().get());
  //std::cerr << "status of recieved message" << message_status << size_status << std::endl;
  // std::string message = temp;
  delete p_length;
  return temp;
}


int broadcast_vector_of_doubles(std::vector<double> * message, int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int length = message->size();
  int * p_length = &length;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  //unsigned short * list = message[0];
  int message_status = 0;
  if (length>0)
    message_status = MPI_Bcast(&(*message)[0], length, MPI_DOUBLE, origin, getOOF_Communicator().get());
  // int message_status = MPI_Bcast((unsigned short *) list, length, MPI_UNSIGNED_SHORT, origin, getOOF_Communicator().get());
  //std::cerr << "status of sent message" << message_status  << size_status << std::endl;
  return message_status && size_status;
}


std::vector<double> *  receive_broadcast_vector_of_doubles(int origin) {
  /* 
     Message is broadcast in two steps.
     First broadcast size of message and then broadcast actual message.
  */
  int * p_length = new int;
  int size_status = MPI_Bcast(p_length, 1, MPI_INT, origin, getOOF_Communicator().get());
  int length = *(p_length);
  std::vector<double> * temp = new std::vector<double>(length);
  int message_status = 0;
  if (length>0)
    message_status = MPI_Bcast(&(*temp)[0], length, MPI_DOUBLE, origin, getOOF_Communicator().get());
  //std::cerr << "status of recieved message" << message_status << size_status << std::endl;
  // std::string message = temp;
  delete p_length;
  return temp;
}


//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\



MPICommunicator::MPICommunicator(){
  MPI_Group mpi_world;
  MPI_Group oof_world;
  int world_size;
  MPI_Comm_size(MPI_COMM_WORLD,&world_size); //get a coppy of the size of the virtual machine
  int * world_ranks = new int [world_size];
  for (int i = 0; i<world_size; i++)
    world_ranks[i] = i; //get a copy of all the ranks
  MPI_Comm_group(MPI_COMM_WORLD, &mpi_world); //get a copy of the world of processes
  MPI_Group_incl(mpi_world, world_size, world_ranks, &oof_world); //pass a list of all the active processors
  MPI_Comm_create(MPI_COMM_WORLD, oof_world, &communicator); //create a new communicator
}

MPICommunicator  & getOOF_Communicator(){
  static MPICommunicator OOF_COMM_WORLD;
  return OOF_COMM_WORLD;
}


