# -*- python -*-
# $RCSfile: analyzePage.py,v $
# $Revision: 1.32 $
# $Author: langer $
# $Date: 2006/02/28 02:48:41 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import utils
from oof2.common.IO import automatic
from oof2.common.IO import output
from oof2.common.IO.GUI import destinationparamwidget
from oof2.common.IO.GUI import fileselector
from oof2.common.IO.GUI import gtkutils
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import outputwidget
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine import analysisdomain
from oof2.engine import analysissample
from oof2.engine.IO import analyze
from oof2.engine.IO import analyzemenu
from oof2.engine.IO.GUI import sampleregclassfactory
import oof2.engine.mesh
import gtk


# A page on which various aspects of the solved mesh can be queried --
# cross-section and statistical outputs will live here, with the
# ability to be put into files, and so forth.

class AnalyzePage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Analysis",
                                 ordering=259,
                                 tip="Query the mesh, examine fields and fluxes.")
        
        mainbox = gtk.VBox()
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        self.meshwidget = whowidget.WhoWidget(oof2.engine.mesh.meshes,
                                              scope=self)
        # The mesh widget callback is not required, because the field
        # and flux widgets in the "output" widget (which are members
        # of a parameter table, which is a component of the
        # OutputWidget) are context-sensitive and update themselves
        # automatically.
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)

        outertable = gtk.Table(rows=2, columns=2)
        outertable.set_homogeneous(1)
        outertable.set_col_spacings(3)
        outertable.set_row_spacings(3)

        mainbox.pack_start(outertable, expand=1, fill=1)



        outputframe = gtk.Frame(label="Output")
        outputframe.set_shadow_type(gtk.SHADOW_IN)
        output_scroll = gtk.ScrolledWindow()
        output_scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        outputframe.add(output_scroll)
        output_box = gtk.VBox()
        output_type_selector_box = gtk.HBox()
        output_box.pack_start(output_type_selector_box, expand=0, fill=0)

        self.scalar_output_button = gtk.RadioButton(label="Scalar")
        self.aggregate_output_button = gtk.RadioButton(
            group=self.scalar_output_button, label="Aggregate")
        output_type_selector_box.pack_start(self.scalar_output_button,
                                            expand=0, fill=0)
        output_type_selector_box.pack_start(self.aggregate_output_button,
                                            expand=0, fill=0)
        self.scalar_output_button.connect("clicked", self.scalar_button_CB)

        # Both aggregate and scalar widgets are built here, and they
        # are switched by show/hide.
        self.scalar_output_button.set_active(1)
        self.scalar_output_obj = outputwidget.ScalarOutputParameterWidget(
            None, scope=self)
        self.aggregate_output_obj = \
                                  outputwidget.AggregateOutputParameterWidget(
            None, scope=self)
        
        self.output_obj = self.scalar_output_obj
        output_box.pack_start(self.scalar_output_obj.gtk, expand=0, fill=0)
        output_box.pack_start(self.aggregate_output_obj.gtk, expand=0, fill=0)
        self.aggregate_output_obj.gtk.hide()
        
        output_scroll.add_with_viewport(output_box)
        outertable.attach(outputframe,0,1,0,1,
                          xoptions=gtk.EXPAND|gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL)


        # The "Operation" frame also contains the widgets for
        # setting the formatting options.  These are here for
        # historical reasons (they used to be params for the Direct
        # operation).
        operationframe = gtk.Frame(label="Operation")
        operationframe.set_shadow_type(gtk.SHADOW_IN)
        op_scroll = gtk.ScrolledWindow()
        op_scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.op_obj = regclassfactory.RegisteredClassFactory(
            analyze.DataOperation.registry, scope=self,
            callback = self.newOperationCB)
        operationframe.add(op_scroll)

        self.formatting_widget = parameterwidgets.ParameterTable(
            analyzemenu.formatting_params)
        
        operation_box = gtk.VBox()
        operation_box.pack_start(self.op_obj.gtk, expand=0, fill=0)
        operation_box.pack_start(self.formatting_widget.gtk, expand=0, fill=0)
        
        op_scroll.add_with_viewport(operation_box)
        
        outertable.attach(operationframe,0,1,1,2,
                          xoptions=gtk.EXPAND|gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL)



        domainframe = gtk.Frame(label="Domain")
        dom_scroll = gtk.ScrolledWindow()
        dom_scroll.set_shadow_type(gtk.SHADOW_IN)
        dom_scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.domain_obj = regclassfactory.RegisteredClassFactory(
            analysisdomain.Domain.registry, scope=self,
            callback = self.newDomainCB)
        domainframe.add(dom_scroll)
        dom_scroll.add_with_viewport(self.domain_obj.gtk)
        outertable.attach(domainframe, 1,2,0,1,
                          xoptions=gtk.EXPAND|gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL)

        sampleframe = gtk.Frame(label="Sampling")
        sampleframe.set_shadow_type(gtk.SHADOW_IN)
        sam_scroll = gtk.ScrolledWindow()
        sam_scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.sample_obj = sampleregclassfactory.SampleRCF(
            analysissample.SampleSet.registry, scope=self,
            callback = self.newSampleCB)
        sampleframe.add(sam_scroll)
        sam_scroll.add_with_viewport(self.sample_obj.gtk)
        outertable.attach(sampleframe, 1,2,1,2,
                          xoptions=gtk.EXPAND|gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL)



        actionframe = gtk.Frame("Destination")
        actionframe.set_shadow_type(gtk.SHADOW_IN)
        mainbox.pack_start(actionframe, expand=0, fill=0, padding=3)
        actionbox = gtk.HBox()
##        actionbox.set_border_width(2)
        actionframe.add(actionbox)

        self.destwidget = destinationparamwidget.DestinationParamWidget()
        self.destwidget.set_value(automatic.automatic)
        actionbox.pack_start(self.destwidget.gtk, expand=1, fill=1, padding=2)
        
        self.go_button = gtkutils.stockButton(gtk.STOCK_EXECUTE, "Go")
        self.go_button.connect("clicked", self.go_buttonCB)
        guitop.top().tooltip(self.go_button, "Send the output to the destination.")
        actionbox.pack_start(self.go_button, fill=1, expand=1, padding=2)

        
        # Whenever fields or fluxes are defined or undefined on the
        # mesh, we need to update the output object widget, and
        # possibly invalidate the currently-displayed data, once we
        # start displaying data.
        switchboard.requestCallback("field defined", self.new_mesh_data)
        switchboard.requestCallback("equation activated", self.new_mesh_data)

        switchboard.requestCallback(("new who", "Mesh"), self.new_mesh)

        switchboard.requestCallback( ('validity', self.domain_obj),
                                     self.sensitize_widgets)
        switchboard.requestCallback( ('validity', self.op_obj),
                                     self.sensitize_widgets)
        switchboard.requestCallback( ('validity', self.scalar_output_obj),
                                     self.sensitize_widgets)
        switchboard.requestCallback( ('validity', self.aggregate_output_obj),
                                     self.sensitize_widgets)
        switchboard.requestCallback( ('validity', self.destwidget),
                                     self.sensitize_widgets)

        # Also, the allowed sampling options may change when the
        # operation or domain change.
        switchboard.requestCallback(self.op_obj, self.update_sampling)
        switchboard.requestCallback(self.domain_obj, self.update_sampling)

        self.update_sampling()
        self.sensitize_widgets()

    def show(self):
        self.gtk.show_all()
        if self.scalar_output_button.get_active():
            self.aggregate_output_obj.gtk.hide()
        else:
            self.scalar_output_obj.gtk.hide()
        
    def sensitize_widgets(self, *args):
        go_sensitive = bool(self.op_obj.isValid() and
                            self.output_obj.isValid() and 
                            self.destwidget.isValid() and
                            self.domain_obj.isValid() and 
                            self.meshwidget.isValid())
        self.go_button.set_sensitive(go_sensitive)

    # GUI callback for the button that switches the widget.  Also
    # switches "self.output_obj", so it can be mindlessly read by the
    # "go"-button callback.
    def scalar_button_CB(self, gtkobj):
        if self.scalar_output_button.get_active():
            self.output_obj = self.scalar_output_obj
            self.aggregate_output_obj.gtk.hide()
            self.scalar_output_obj.show()
        else:
            self.output_obj = self.aggregate_output_obj
            self.scalar_output_obj.gtk.hide()
            self.aggregate_output_obj.show()
        self.sensitize_widgets()
        
    # Switchboard, "field defined" and "equation activated"
    def new_mesh_data(self, meshname, *args):
        if meshname == self.meshwidget.get_value():
            self.scalar_output_obj.treeCB()
            self.aggregate_output_obj.treeCB()

    # Switchboard, ("new who", "Mesh")
    def new_mesh(self, mesh):
        path = labeltree.makePath(mesh)
        self.meshwidget.set_value(path)
        self.sensitize_widgets()

    # Callback from "operation" registered callback.
    def newOperationCB(self, registration):
        self.sensitize_widgets()

    def newDomainCB(self, registration):
        self.sensitize_widgets()

    def newSampleCB(self, registration):
        self.sensitize_widgets()
    
    # When either the domain or the operation change, the allowed
    # samplings can change -- update the widget here.
    def update_sampling(self, *args):
        domain_reg = self.domain_obj.getRegistration()
        op_reg = self.op_obj.getRegistration()

        if domain_reg:
            sample_types = domain_reg.sample_types
        else:
            sample_types = []

        if op_reg:
            directness = op_reg.direct
        else:
            directness = None

        self.sample_obj.set_defaults()
        reg = self.sample_obj.getRegistration()
        regname = None
        if reg:
            regname = reg.name()
        self.sample_obj.setSampleCriteria(sample_types, directness)

        self.sample_obj.update(analysissample.SampleSet.registry,
                               obj=regname)


    # "Go" button callback -- fill in the basic parameters, and
    # perform the action.
    def go_buttonCB(self, gtkobj):
        op_reg = self.op_obj.get_value()
        regname = op_reg.getRegistration().name()
        menuitem = analyzemenu.ops_menu.getItem(
            utils.space2underscore(regname))

        self.formatting_widget.get_values()

        menuitem.callWithDefaults(mesh=self.meshwidget.get_value(),
                                  data=self.output_obj.get_value(),
                                  domain=self.domain_obj.get_value(),
                                  sampling=self.sample_obj.get_value(),
                                  destination=self.destwidget.get_value())
        

analyzepage = AnalyzePage()
