# -*- python -*-
# $RCSfile: meshdataGUI.py,v $
# $Revision: 1.24 $
# $Author: langer $
# $Date: 2006/03/10 21:54:14 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

## Window for displaying mesh data.

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import primitives
from oof2.common import utils
from oof2.common.IO import gfxmanager
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import quit
from oof2.common.IO.GUI import subWindow
from oof2.common.IO.GUI import whowidget
from oof2.common.IO.GUI import widgetscope
from oof2.engine import mesh
from oof2.engine.IO.GUI import outputvalwidgets
import gtk
import oof2.common.IO.output

allMeshDataWindows = []

mdWindowMenu = mainmenu.OOF.Windows.addItem(oofmenu.OOFMenuItem("Mesh_Data"))

## TODO: This isn't really a meshviewer problem, but it's also true
## that the node or element is drawn at the undisplaced position,
## which can be disturbing if the mesh displayed is at the actual or
## enhanced positions.  

class MeshDataGUI(widgetscope.WidgetScope):
    count = 1
    def __init__(self, gfxwindow, position, output=None):
        mainthread.runBlock(self.__init__thread, (gfxwindow, position, output))

    def __init__thread(self, gfxwindow, position, output):
        allMeshDataWindows.append(self)
        widgetscope.WidgetScope.__init__(self, None)

        current_count = MeshDataGUI.count
        MeshDataGUI.count += 1
        self._name = "Mesh_Data_%d" % current_count
        self.output = output
        self.position = position
        self.sbcallbacks = []
        self.gsbcallbacks = []          # callbacks from a specific gfx window

        self.outputparam = \
                     oof2.common.IO.output.AggregateOutputParameter('output')

        # Although it's not displayed, we need a mesh widget in the
        # widgetscope, or the OutputParameterWidget won't work.
        # TODO LATER: Is this ugly, or what?
        self.meshwidget = whowidget.WhoWidget(mesh.meshes, scope=self)

        self.gtk = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.gtk.set_title(utils.underscore2space(self._name))
        self.mainbox = gtk.VBox()
        self.gtk.add(self.mainbox)

        # Put this window into the Windows menu.  The menu item can't
        # be logged, since the creation and operation of the window
        # aren't logged, so scripts shouldn't refer to it at all.
        mainmenu.OOF.Windows.Mesh_Data.addItem(oofmenu.OOFMenuItem(
            self._name,
            no_log=1,
            help="Raise Mesh Data window %d." % current_count,
            callback=self.raiseWindow))

        expander = gtk.Expander("Source")
        self.mainbox.pack_start(expander)
        expander.set_expanded(1)
        
        self.table = gtk.Table(rows=5, columns=2)
        expander.add(self.table)

        label = gtk.Label("Source Window:")
        label.set_alignment(1.0, 0.5)
        eventbox = gtk.EventBox()
        eventbox.add(label)
        self.table.attach(eventbox, 0,1, 0,1, xpadding=3, xoptions=gtk.FILL)
        guitop.top().tooltip(
            eventbox,
            "Display data for mouse clicks in this Graphics window.")

        self.gfxWindowChooser = chooser.ChooserWidget([],
                                                      callback=self.chooserCB)
        self.table.attach(self.gfxWindowChooser.gtk, 1,2, 0,1,
                     xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)


        label = gtk.Label("Mesh:")
        label.set_alignment(1.0, 0.5)
        eventbox = gtk.EventBox()
        eventbox.add(label)
        self.table.attach(eventbox, 0,1, 1,2,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        guitop.top().tooltip(
            eventbox,
            "Data is displayed for values on this mesh.")

        self.meshtext = gtk.Entry()
        self.meshtext.set_editable(0)
        self.meshtext.set_size_request(12*guitop.top().charsize, -1)
        self.table.attach(self.meshtext, 1,2, 1,2,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)

        label = gtk.Label("Mouse x:")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 2,3,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        self.xtext = gtk.Entry()
        self.xtext.set_size_request(12*guitop.top().digitsize, -1)
        self.table.attach(self.xtext, 1,2, 2,3,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)
        self.xsignal = self.xtext.connect('changed', self.posChangedCB)

        label = gtk.Label("Mouse y:")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 3,4,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        self.ytext = gtk.Entry()
        self.ytext.set_size_request(12*guitop.top().digitsize, -1)
        self.table.attach(self.ytext, 1,2, 3,4,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)
        self.ysignal = self.ytext.connect('changed', self.posChangedCB)

        label = gtk.Label("Output:")
        label.set_alignment(1.0, 0.5)
        eventbox = gtk.EventBox()
        eventbox.add(label)
        self.table.attach(eventbox, 0,1, 4,5,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        guitop.top().tooltip(eventbox, "Choose which data is displayed.")
        
        self.outputwidget = self.outputparam.makeWidget(scope=self)
        self.table.attach(self.outputwidget.gtk, 1,2, 4,5,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)
        switchboard.requestCallback(self.outputwidget, self.outputwidgetCB)

        hbox = gtk.HBox()
        self.mainbox.pack_start(hbox, expand=1, fill=1, padding=5)
        frame = gtk.Frame("Data")
        frame.set_shadow_type(gtk.SHADOW_IN)
        hbox.pack_start(frame, expand=1, fill=1, padding=5)
        vbox = gtk.VBox()
        frame.add(vbox)
        self.databox = gtk.HBox()
        vbox.pack_start(self.databox, expand=1, fill=1, padding=3)
        self.datawidget = None       # set by updateData

        # Buttons at the bottom of the window
        align = gtk.Alignment(xalign=0.5)
        self.mainbox.pack_start(align, expand=0, fill=0, padding=3)
        hbox = gtk.HBox()
        align.add(hbox)
        # Freeze button
        self.freezeflag = 0
        self.freezebutton = gtk.CheckButton('Freeze')
        hbox.pack_start(self.freezebutton, expand=0, fill=0, padding=3)
        self.freezebutton.set_active(self.freezeflag)
        self.freezebutton.connect('clicked', self.freezeButtonCB)
        guitop.top().tooltip(self.freezebutton,
                        "Prevent the data in this window from being updated.")

        # Close button
        self.closeButton = gtk.Button(stock=gtk.STOCK_CLOSE)
        self.closeButton.connect('clicked', self.closeButtonCB)
        hbox.pack_end(self.closeButton, expand=0, fill=0, padding=3)

        self.gtk.connect('destroy', self.destroyCB)

        self.updateGfxWindowChooser()
        self.gfxWindowChooser.set_state(gfxwindow.name)
        if position is not None:
            self.updatePosition(position)
        self.currentMesh = None
        self.updateMesh()

        self.setupSwitchboard()         # gfx window dependent callbacks
        self.sbcallbacks += [
            switchboard.requestCallback('open graphics window',
                                        self.gfxwindowChanged),
            switchboard.requestCallback('close graphics window',
                                        self.gfxwindowChanged)
            ]

        
        self.gtk.show_all()

    def raiseWindow(self, menuitem):
        self.gtk.window.raise_()

    ###################

    # Gfx window management

    def gfxwindowChanged(self, window): # sb callback, window opened or closed
        self.updateGfxWindowChooser()

    def updateGfxWindowChooser(self):
        self.gfxWindowChooser.update(
            [w.name for w in gfxmanager.gfxManager.windows])

    def chooserCB(self, chsr, gfxwindowname):
        self.setupSwitchboard()
        self.updateMesh()               # calls updateData if needed

    def setupSwitchboard(self):
        map(switchboard.removeCallback, self.gsbcallbacks)
        window = self.currentGfxWindow()
        if window is not None:
            self.gsbcallbacks = [
                switchboard.requestCallback((window, "layers changed"),
                                            self.layersChangedCB),
                switchboard.requestCallback((window, "meshinfo click"),
                                            self.updatePosition)
            ]
        else:
            self.gsbcallbacks = []

    def currentGfxWindow(self):
        name = self.gfxWindowChooser.get_value()
        if name:
            return gfxmanager.gfxManager.getWindow(name)

    ##############

    def closeButtonCB(self, button):
        self.gtk.destroy()

    def destroyCB(self, *args):         # gtk callback
        mainmenu.OOF.Windows.Mesh_Data.removeItem(self._name)
        map(switchboard.removeCallback, self.sbcallbacks)
        map(switchboard.removeCallback, self.gsbcallbacks)
        allMeshDataWindows.remove(self)

    ##############

    def freezeButtonCB(self, button):
        self.freezeflag = button.get_active()
        self.table.set_sensitive(not self.freezeflag)
        if not self.freezeflag:
            self.updateMesh()
            self.updatePosition(self.position)
            self.updateData()

    ##############

    def layersChangedCB(self):
        self.updateMesh()
        self.updateData()

    def outputwidgetCB(self, interactive): # chosen output has changed
        self.updateData()

    def posChangedCB(self, gtkobj):     # text edited in x & y gtk.Entries
        try:
            self.position = primitives.Point(_getval(self.xtext),
                                             _getval(self.ytext))
        except:
            # The user has typed something that can't be evaluated,
            # but maybe there's more typing to come. Ignore the error.
            if self.datawidget:
                self.datawidget.gtk.set_sensitive(0)
        else:
            self.updateData()

    ##############

    def updateMesh(self):
        debug.mainthreadTest()
        if not self.freezeflag:
            window = self.currentGfxWindow()
            newmesh = None
            if window:
                newmesh = window.topwho('Mesh')
                if newmesh:
                    self.meshtext.set_text(newmesh.path())
                else:
                    self.meshtext.set_text('<No Mesh in window!>')
            else:
                self.meshtext.set_text('---')
            if newmesh is not self.currentMesh:
                self.currentMesh = newmesh
                self.updateData()
            if newmesh is not None:
                self.meshwidget.set_value(newmesh.path())
            else:
                self.meshwidget.set_value(None)

    def updatePosition(self, position):
        debug.mainthreadTest()
        if not self.freezeflag:
            if position != self.position:
                self.position = position
                self.updateData()
            self.xtext.handler_block(self.xsignal)
            self.ytext.handler_block(self.ysignal)
            if position is not None:
                # Strip blanks to the right of the number so that they
                # don't get in the way when the user edits the
                # position.
                self.xtext.set_text(("%-13.6g" % position.x).rstrip())
                self.ytext.set_text(("%-13.6g" % position.y).rstrip())
            else:
                # Probably not required -- position is initially None,
                # and having once been set, probably can't become
                # None.
                self.xtext.set_text("")
                self.ytext.set_text("")
            self.xtext.handler_unblock(self.xsignal)
            self.ytext.handler_unblock(self.ysignal)
                

    def updateData(self):
        if not self.freezeflag:
            if self.datawidget:
                self.datawidget.destroy()
            if self.currentMesh is not None and self.position is not None:
                op = self.outputwidget.get_value()
                if op is not None and not op.incomputable(self.currentMesh):
                    element = self.currentMesh.enclosingElement(self.position)
                    if element is not None:
                        masterpos = element.to_master(self.position.x,
                                                      self.position.y)
                        val = op.evaluate(self.currentMesh.getObject(),
                                          [element], [[masterpos]])[0]
                        self.datawidget = outputvalwidgets.makeWidget(val)
                        self.databox.pack_start(self.datawidget.gtk,
                                                expand=1, fill=1, padding=3)
                        self.datawidget.show()


def _getval(widget):
    text = widget.get_text().lstrip()
    if text:
        return utils.OOFeval(text)
    return 0.0

###############################
        
def openMeshData(gfxwindow, position, output=None):
    MeshDataGUI(gfxwindow, position, output)

