# -*- python -*-
# $RCSfile: skeletongroupwidgets.py,v $
# $Revision: 1.49 $
# $Author: langer $
# $Date: 2005/08/08 21:39:35 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common.IO import selectionplaceholder
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import whowidget
from oof2.engine import boundarybuilder
from oof2.engine import boundarymodifier
from oof2.engine import mesh
from oof2.engine import skeletoncontext
from oof2.engine.IO import skeletongroupparams
from oof2.engine.IO import meshbdymenu
from oof2.engine.IO.GUI import bdymodparamwidget

# Special widgets for the SkeletonGroup parameters.

# The "defaults" argument is used by the "Aggregate" widget, to
# provide a mechanism for picking sets of SkeletonSelectables which
# are not groups.  Currently only "<selection>" is available, but
# other examples could include "all", "none", and possibly others.

class SkeletonGroupWidget(parameterwidgets.ParameterWidget):
    def __init__(self, groups=[], defaults=[], scope=None):
        self.defaults = defaults
        self.widget = chooser.ChooserWidget(groups, self.selectCB)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.skelwidget = scope.findWidget(
            lambda w: isinstance(w, whowidget.WhoWidget)
            and w.whoclass is skeletoncontext.skeletonContexts)
        if self.skelwidget is not None:
            self.sbcallbacks = [
                switchboard.requestCallback(self.skelwidget, self.skelwidgetCB)]
        else:
            self.meshwidget = scope.findWidget(
                lambda w: isinstance(w, whowidget.WhoWidget)
                and w.whoclass is mesh.meshes)
            self.sbcallbacks = [
                switchboard.requestCallback(self.meshwidget, self.skelwidgetCB)]
        self.update()
        self.sbcallbacks += [
            switchboard.requestCallback("groupset member added", self.grpCB),
            switchboard.requestCallback("groupset member renamed", self.grpCB),
            switchboard.requestCallback("groupset changed", self.grpCB)
            ]
        self.widgetChanged(self.widget.nChoices() > 0, interactive=0)

    def get_value(self):
        return self.widget.get_value()

    def selectCB(self, gtkobj, result):
        self.widgetChanged(self.widget.nChoices() > 0, interactive=1)

    def skelwidgetCB(self, interactive):
        self.update()
        self.widgetChanged(self.widget.nChoices() > 0, interactive)
    def grpCB(self, *args, **kwargs):
        self.update()
        self.widgetChanged(self.widget.nChoices() > 0, interactive=0)

    def getSkeleton(self):
        if self.skelwidget is not None:
            skelname = self.skelwidget.get_value()
        else:
            skelname = self.meshwidget.get_value(depth=2)
        try:
            return skeletoncontext.skeletonContexts[skelname]
        except KeyError:
            return None

    def update(self):
        self.redraw(self.getSkeleton())
##        try:
##            skel = skeletoncontext.skeletonContexts[self.skelwidget.get_value()]
##        except KeyError:
##            self.redraw(None)
##        else:
##            self.redraw(skel)

    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        parameterwidgets.ParameterWidget.cleanUp(self)

class SkeletonAggregateWidget(SkeletonGroupWidget):
    def get_value(self):
        rval = self.widget.get_value()
        if rval == selectionplaceholder.selectionIDstring:
            return selectionplaceholder.selectionPlaceHolder
        return rval

# Dictionary, indexed by widget class, of the function to use to
# get a segment set from the relevant aggregate name.
segmenter = {}

class NodeGroupWidget(SkeletonGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups, scope=scope)

    def redraw(self, skeletoncontext):
        if skeletoncontext:
            self.widget.update(self.defaults +
                               skeletoncontext.nodegroups.allGroups())
        else:
            self.widget.update(self.defaults)

def _makeNodeGroupWidget(self, scope=None):   # "self" is the param instance.
    return NodeGroupWidget(self, scope=scope)

skeletongroupparams.NodeGroupParameter.makeWidget = _makeNodeGroupWidget


class SkeletonAggregateWidget(SkeletonGroupWidget):
    def get_value(self):
        rval = self.widget.get_value()
        if rval == selectionplaceholder.selectionIDstring:
            return selectionplaceholder.selectionPlaceHolder
        return rval

    
class NodeAggregateWidget(SkeletonAggregateWidget, NodeGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups,
                                     defaults=[selectionplaceholder.selectionIDstring],
                                     scope=scope)
        
def _makeNodeAggregateWidget(self, scope=None):
    return NodeAggregateWidget(self, scope=scope)

skeletongroupparams.NodeAggregateParameter.makeWidget = _makeNodeAggregateWidget

segmenter[NodeAggregateWidget]=boundarybuilder.segments_from_node_aggregate

####

class SegmentGroupWidget(SkeletonGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups, scope=scope)

    def redraw(self, skeletoncontext):
        if skeletoncontext:
            self.widget.update(self.defaults +
                               skeletoncontext.segmentgroups.allGroups() )
        else:
            self.widget.update(self.defaults)

def _makeSegmentGroupWidget(self, scope=None):
    return SegmentGroupWidget(self, scope=scope)

skeletongroupparams.SegmentGroupParameter.makeWidget = _makeSegmentGroupWidget



class SegmentAggregateWidget(SkeletonAggregateWidget, SegmentGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups,
                                     defaults=[selectionplaceholder.selectionIDstring],
                                     scope=scope)
        
def _makeSegmentAggregateWidget(self, scope=None):
    return SegmentAggregateWidget(self, scope=scope)

skeletongroupparams.SegmentAggregateParameter.makeWidget = _makeSegmentAggregateWidget

segmenter[SegmentAggregateWidget]=boundarybuilder.segments_from_seg_aggregate


# Special context-aware segment aggregate widget for use in boundary
# modifiers.  This widget can tell whether or not the intended
# modification to the current boundary of the current segment will
# result in a sequence-able segment set, and will only report itself
# as "valid" if this is true.  Note that it is necessary to actually
# attempt the operation, sequence-ability cannot be deduced from the
# segment aggregate alone -- it may "tee" into the boundary, creating
# a branch, and fail even though the aggregate itself can be
# sequenced, or it may consist of two separate branches at the two
# end-points of the boundary, and thus succeed even though the
# aggregate itself cannot be sequenced.
# 
# This widget lives in a modal dialog box, and is a subwidget for a
# parameter to the boundary modifier, so the only real "dynamism" is
# that the value of the widget itself (i.e. the aggregate selection)
# can change.  We can get the skeleton and the modifier at init-time.

class BdyModSegmentAggregateWidget(SegmentAggregateWidget, SegmentGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups,
                                     defaults=[selectionplaceholder.selectionIDstring],
                                     scope=scope)
        # self.skelwidget has been set by the parent.  The modifier
        # widget will not change during our lifetime, since changing
        # it causes a new aggregate widget to be created.
        self.modifierwidget = scope.findWidget(
            lambda w: isinstance(w, bdymodparamwidget.BoundaryModParamWidget))
        
        # The modifierwidget's scope's parent is the ParameterDialog
        # box, which has the boundary name.
        self.bdy_name = self.modifierwidget.scope.parent.boundaryname
        self.widgetChanged(self.local_validity(), interactive=0)
        # self.local_validity()
        
    def selectCB(self, gtkobj, result):
         #self.local_validity()
        self.widgetChanged(self.local_validity(), interactive=1)

        
    def local_validity(self):
        if self.widget.nChoices()>0 and self.modifierwidget \
               and self.skelwidget:
            grp = self.get_value()
            mod_reg = self.modifierwidget.getRegistration()
            mod_obj = mod_reg(group=grp)
            skelctxt = skeletoncontext.skeletonContexts[
                self.skelwidget.get_value()]
            return mod_obj.attempt(skelctxt, self.bdy_name)

        

def _makeBMSegmentAggregateWidget(self, scope=None):
    return BdyModSegmentAggregateWidget(self, scope=scope)

boundarymodifier.BdyModSegmentAggregateParameter.makeWidget = _makeBMSegmentAggregateWidget

segmenter[BdyModSegmentAggregateWidget]=boundarybuilder.segments_from_seg_aggregate



####

class ElementGroupWidget(SkeletonGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups, defaults=[], scope=scope)

    def redraw(self, skeletoncontext):
        if skeletoncontext:
            self.widget.update(self.defaults +
                               skeletoncontext.elementgroups.allGroups())
        else:
            self.widget.update(self.defaults)

def _makeElementGroupWidget(self, scope=None):
    return ElementGroupWidget(self, scope=scope)

skeletongroupparams.ElementGroupParameter.makeWidget = _makeElementGroupWidget

class ElementAggregateWidget(SkeletonAggregateWidget, ElementGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups,
                                     defaults=[selectionplaceholder.selectionIDstring],
                                     scope=scope)
        

def _makeElementAggregateWidget(self, scope=None):
    return ElementAggregateWidget(self, scope=scope)

skeletongroupparams.ElementAggregateParameter.makeWidget = _makeElementAggregateWidget

segmenter[ElementAggregateWidget]=boundarybuilder.segments_from_el_aggregate

# ## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ## #


# Scope-aware widget for picking the boundary from the local skeleton
# context.

class SkeletonBoundaryWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, boundaries=[], scope=None):
        self.widget = chooser.ChooserWidget(boundaries, self.selectCB)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.skelwidget = scope.findWidget(
            lambda w: isinstance(w, whowidget.WhoWidget)
            and (w.whoclass is skeletoncontext.skeletonContexts))
        self.update()

        self.sbcallbacks = [
            switchboard.requestCallback(self.skelwidget, self.skelwidgetCB),
            switchboard.requestCallback("new boundary configuration",
                                        self.update)
            ]
        if param.value is not None:
            self.widget.set_state(param.value)
        self.widgetChanged(self.widget.nChoices() > 0, interactive=0)            

    def get_value(self):
        return self.widget.get_value()
    def selectCB(self, gtkobj, result):
        self.widgetChanged(self.widget.nChoices() > 0, interactive=1)
    def skelwidgetCB(self, interactive):
        self.update()
        self.widgetChanged(self.widget.nChoices() > 0, interactive)

    def update(self, *args, **kwargs):
        try:
            skel = skeletoncontext.skeletonContexts[
                self.skelwidget.get_value()]
        except KeyError:
            skel = None

        self.redraw(skel)

    def redraw(self, skel):
        if skel:
            self.widget.update(skel.allBoundaryNames())
        else:
            self.widget.update([])

    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        parameterwidgets.ParameterWidget.cleanUp(self)
        
def _makeSkeletonBoundaryWidget(self, scope=None):
    return SkeletonBoundaryWidget(self, scope=scope)

skeletongroupparams.SkeletonBoundaryParameter.makeWidget = \
                                                _makeSkeletonBoundaryWidget


# ## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ## #

from oof2.engine import profile

class ProfileNameWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, profiles=[], scope=None):
        self.widget = chooser.ChooserWidget(profiles, self.selectCB)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.update(param.value)
        self.widgetChanged(self.widget.nChoices() > 0, interactive=0)
        # Widget needs to notice both when profiles are changed and when
        # they're renamed -- "changed" signal is for addition/removal of
        # profiles, whereas "renamed" is of course for rename operations.
        self.sbcallbacks = [
            switchboard.requestCallback("profiles changed", self.profChangeCB),
            switchboard.requestCallback("profile renamed", self.profRenameCB)
            ]

    def get_value(self):
        return self.widget.get_value()
    
    def selectCB(self, gtkobj, result):
        self.widgetChanged(self.widget.nChoices() > 0, interactive=1)
    def profChangeCB(self, name):
        self.update(name)
        self.widgetChanged(self.widget.nChoices() > 0, interactive=0)
    def profRenameCB(self, oldname, newname):
        self.update(newname)
        self.widgetChanged(self.widget.nChoices() > 0, interactive=0)

    def update(self, name):
        self.widget.update(profile.AllProfiles.keys())
        if name:
            self.widget.set_state(name)

    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        parameterwidgets.ParameterWidget.cleanUp(self)
        
def _makeProfileNameWidget(self, scope=None):
    return ProfileNameWidget(self, scope=scope)

profile.ProfileNameParameter.makeWidget = _makeProfileNameWidget
