# -*- python -*-
# $RCSfile: subproblemmenu.py,v $
# $Revision: 1.24 $
# $Author: reida $
# $Date: 2006/02/14 19:36:54 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import devel
from oof2.SWIG.common import ooferror
from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import field
from oof2.SWIG.engine import solverdriver
from oof2.SWIG.engine import subproblem
from oof2.common import debug
from oof2.common import labeltree
from oof2.common.IO import automatic
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import parameter
from oof2.common.IO import whoville
from oof2.engine import fieldinit
from oof2.engine import timedrivers
from oof2.engine.IO import meshparameters
from oof2.engine.IO import skeletongroupparams
import oof2.engine.mesh
import oof2.engine.subproblemcontext
OOF = mainmenu.OOF

if devel.devel()>=2:
    subproblemMenu = OOF.addItem(oofmenu.OOFMenuItem(
        'SubProblem',
        help="Tools for creating and solving sections of meshes",
        cli_only=1))
    # Look for an enclosing mesh parameter -- if not found, use the
    # enclosing skeleton parameter.  Mesh copying needs the first case,
    # new mesh construction needs the second.

    def sproblemNameResolver(param, startname):
        if param.automatic():
            basename = 'subproblem'
        else:
            basename = startname

        try:
            spname = param.group['subproblem'].value
        except IndexError:
            meshname = param.group['mesh'].value
            meshpath = labeltree.makePath(meshname)
        else:
            meshpath = labeltree.makePath(spname)[:-1]

        return oof2.engine.subproblemcontext.subproblems.uniqueName(meshpath + [basename])


    def __new_problem(menuitem, mesh, name):
        meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
        meshcontext.reserve()
        try:
            pblm = subproblem.SubProblem(name)
            if pblm is not None:
                meshpath = labeltree.makePath(mesh)
                oof2.engine.subproblemcontext.subproblems.add(
                    meshpath+[name],
                    pblm
                    )
            problemcontext = oof2.engine.subproblemcontext.subproblems[meshpath+[name]]
            problemcontext.setMesh(meshcontext)
            switchboard.notify("new subproblem", name, mesh)
        finally:
            meshcontext.cancel_reservation()


    subproblemMenu.addItem(oofmenu.OOFMenuItem(
        'New',
        callback=__new_problem,
        threadable= oofmenu.THREADABLE,
        params=parameter.ParameterGroup(
        whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                              tip=parameter.emptyTipString),
        parameter.AutomaticNameParameter('name', value=automatic.automatic,
                                         resolver=sproblemNameResolver,
                                         tip="Name of the new SubProblem")
        ),
        help="Define new subproblem"
        )
                       )
    def __new_all_subproblem(menuitem, mesh, name):
        meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
        meshcontext.reserve()
        try:
            pblm = subproblem.SubProblemAll(name)
            if pblm is not None:
                meshpath = labeltree.makePath(mesh)
                oof2.engine.subproblemcontext.subproblems.add(
                    meshpath+[name],
                    pblm
                    )
            problemcontext = oof2.engine.subproblemcontext.subproblems[meshpath+[name]]
            problemcontext.setMesh(meshcontext)
            switchboard.notify("new subproblem", name, mesh)
        finally:
            meshcontext.cancel_reservation()

    def __activate_equation(menuitem, subproblem, equation):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.activateEquation(equation) ## equation is a real equation
            switchboard.notify('equation activated', subproblem, equation.name(), 1)
            ## subproblemcontext.autoenableBCs()
        finally:
            subproblemcontext.cancel_reservation()


    def __deactivate_equation(menuitem, subproblem, equation):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.deactivateEquation(equation) ## equation is a real equation
            switchboard.notify('equation activated', subproblem, equation.name(), 0)
            ## subproblemcontext.autoenableBCs()
        finally:
            subproblemcontext.cancel_reservation()



    def __activate_kinetics(menuitem, subproblem, equation):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.activateKinetics(equation) ## equation is a real equation
            switchboard.notify('kinetics activated', subproblem, equation.name(), 1)
        finally:
            subproblemcontext.cancel_reservation()

    def __deactivate_kinetics(menuitem, subproblem, equation):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.deactivateKinetics(equation) ## equation is a real equation
            switchboard.notify('kinetics activated', subproblem, equation.name(), 0)
        finally:
            subproblemcontext.cancel_reservation()

    eqnmenu = subproblemMenu.addItem(oofmenu.OOFMenuItem('Equation',
                                                   help='Activate equations'))
    eqnmenu.addItem(oofmenu.OOFMenuItem(
        'Activate',
        callback=__activate_equation,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.EquationParameter('equation',
                                                 tip=parameter.emptyTipString)
                ],
        help="Activate an equation in subproblem"
        ))

    eqnmenu.addItem(oofmenu.OOFMenuItem(
        'Deactivate',
        callback=__deactivate_equation,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.EquationParameter('equation',
                                                 tip=parameter.emptyTipString)
                ],
        help="Deactivate an equation in subproblem"
        ))


    eqnmenu.addItem(oofmenu.OOFMenuItem(
        'ActivateKinetics',
        callback=__activate_kinetics,
        threadable=oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.EquationParameter('equation',
                                                 tip=parameter.emptyTipString)
                ]
        ))

    eqnmenu.addItem(oofmenu.OOFMenuItem(
        'DeactivateKinetics',
        callback=__deactivate_kinetics,
        threadable=oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.EquationParameter('equation',
                                                 tip=parameter.emptyTipString)
        ]
        ))

    def __activate_dynamics(menuitem, subproblem, equation):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.activateDynamics(equation) ## equation is a real equation
            switchboard.notify('dynamics activated', subproblem, equation.name(), 1)
        finally:
            subproblemcontext.cancel_reservation()

    def __deactivate_dynamics(menuitem, subproblem, equation):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.deactivateDynamics(equation) ## equation is a real equation
            switchboard.notify('dynamics activated', subproblem, equation.name(), 0)
        finally:
            subproblemcontext.cancel_reservation()

    eqnmenu.addItem(oofmenu.OOFMenuItem(
        'ActivateDynamics',
        callback=__activate_dynamics,
        threadable=oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.EquationParameter('equation',
                                                 tip=parameter.emptyTipString)
                ]
        ))

    eqnmenu.addItem(oofmenu.OOFMenuItem(
        'DeactivateDynamics',
        callback=__deactivate_dynamics,
        threadable=oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.EquationParameter('equation',
                                                 tip=parameter.emptyTipString)
        ]
        ))
    def __activate_field(menuitem, subproblem, field):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:

            prob = subproblemcontext.getObject()
            prob.activateField(field)## field is a real field
            switchboard.notify("field activated", subproblem, field.name(), 1)
            ## subproblemcontext.autoenableBCs()
        finally:
            subproblemcontext.cancel_reservation()

    def __deactivate_field(menuitem, subproblem, field):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.deactivateField(field) ## field is a real field
            switchboard.notify("field activated", subproblem, field.name(), 0)
            ## subproblemcontext.autoenableBCs()
        finally:
            subproblemcontext.cancel_reservation()

    fieldmenu = subproblemMenu.addItem(oofmenu.OOFMenuItem(
        'Field',
        help='Define and activate Fields.'
        ))

    fieldmenu.addItem(oofmenu.OOFMenuItem(
        "Activate",
        threadable=oofmenu.THREADABLE,
        callback=__activate_field,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
        ],
        help="Activate a Field.  The solver finds the values of active Fields."
        ))

    fieldmenu.addItem(oofmenu.OOFMenuItem(
        'Deactivate',
        threadable=oofmenu.THREADABLE,
        callback=__deactivate_field,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
                ],
        help="Deactivate a Field.  The solver finds the values of active Fields.",
        ))


    def __in_plane_field(menuitem, subproblem, field):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:

            prob = subproblemcontext.getObject()
            prob.inPlaneField(field)## field is a real field
            switchboard.notify("field inplane", subproblem, field.name(), 1)
        finally:
            subproblemcontext.cancel_reservation()

    def __out_of_plane_field(menuitem, subproblem, field):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.outOfPlaneField(field) ## field is a real field
            switchboard.notify("field inplane", subproblem, field.name(), 0)
        finally:
            subproblemcontext.cancel_reservation()



    fieldmenu.addItem(oofmenu.OOFMenuItem(
        "In_Plane",
        threadable=oofmenu.THREADABLE,
        callback=__in_plane_field,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
        ],
        help="Activate a Field.  The solver finds the values of active Fields."
        ))

    fieldmenu.addItem(oofmenu.OOFMenuItem(
        'Out_of_Plane',
        threadable=oofmenu.THREADABLE,
        callback=__out_of_plane_field,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
                ],
        help="Deactivate a Field.  The solver finds the values of active Fields.",
        ))



    #######################################


    # Copy the field state (definitions, active-ness, planarity) of one
    # mesh into another.
    def _copyFieldState(menuitem, source, target):
        if source == target:
            return ## already copied
        source_spblm = oof2.engine.subproblemcontext.subproblems[source]
        target_spblm = oof2.engine.subproblemcontext.subproblems[target]

    ##    ## First transfer the DEFINED fields (fields are defined in mesh)
    ##    source_mesh = source_spblm.getMesh()
    ##    target_mesh = target_spblm.getMesh()
    ##    source_mesh.begin_reading()
    ##    print "source mesh begin reading"
    ##    target_mesh.reserve()
    ##    print "target mesh reserved"
    ##    target_mesh.begin_writing()
    ##    print "target mesh begin writing"
    ##    try:
    ##        source_obj = source_mesh.getObject()
    ##        target_obj = target_mesh.getObject()
    ##        source_fields = source_obj.all_fields()
    ##        target_fields = target_obj.all_fields()
    ##        if source_obj is target_obj:
    ##            raise ooferror.ErrUserError('Source and target must differ!')
    ##        # Undefine all the fields in the target that are not in the source.
    ##        for f in target_fields:
    ##            if not source_obj.is_defined_field(f):
    ##                target_obj.undefine_field(f)
    ##                switchboard.notify("field defined", target, f.name(), 0)
    ##        for f in source_fields:
    ##            # Definition.
    ##            if not target_obj.is_defined_field(f):
    ##                target_obj.define_field(f)
    ##                switchboard.notify("field defined", target, f.name(), 1)
    ##            try:
    ##                initializer = source_mesh.initializers[f]
    ##            except KeyError:
    ##                pass
    ##            else:
    ##                target_mesh.initialize_field(f, initializer)
    ##                switchboard.notify("field initialized")
    ##    finally:
    ##        source_mesh.end_reading()
    ##        print "end source mesh reading"
    ##        target_mesh.end_writing()
    ##        print "end target mesh writing"
    ##        target_mesh.cancel_reservation()
    ##        print "end target mesh reservation"
    ##        target_mesh.changed()
    ##        print "target mesh changed"

        ## Next addresss the active fields
        source_spblm.begin_reading()
        target_spblm.reserve()
        target_spblm.begin_writing()
        try:
            source_obj = source_spblm.getObject()
            target_obj = target_spblm.getObject()
            ## fields from source and target
            source_fields = source_obj.active_fields
            target_fields = target_obj.active_fields
            ## out-of-plane fields from source and target
            source_oup = source_obj.in_plane_fields
            target_oup = target_obj.in_plane_fields
            ## check we are not dealing with same subproblem
            if source_obj is target_obj:
                ## if you made it this far, raise an exception
                raise ooferror.ErrUserError('Source and target must differ!')
            for f in field.allFields:
                # Activation
                ## f = target_fields[fieldname]
                if source_obj.is_active_field(f):
                    if not target_obj.is_active_field(f):
                        target_obj.activateField(f)
                        switchboard.notify("field activated", target, f.name(), 1)
                else:
                    if target_obj.is_active_field(f):
                        target_obj.deactivateField(f)
                        switchboard.notify("field activated", target, f.name(), 0)

                # Planarity
            for f in field.allFields:
                ## f = target_oup[fieldname]
                if source_obj.is_in_plane(f):
                    if not target_obj.is_in_plane(f):
                        target_obj.inPlaneField(f)
                        switchboard.notify("field inplane", target, f.name(), 1)
                else:
                    if target_obj.is_in_plane(f):
                        target_obj.outOfPlaneField(f)
                        switchboard.notify("field inplane", target, f.name(), 0)


        finally:
            source_spblm.end_reading()
            target_spblm.end_writing()
            target_spblm.cancel_reservation()
            switchboard.notify("redraw")
        ## oof2.engine.subproblemcontext.subproblems[target].changed()
    fieldmenu.addItem(oofmenu.OOFMenuItem(
        'Copy_Field_State',
        callback=_copyFieldState,
        threadable=oofmenu.THREADABLE,
        params=[whoville.WhoParameter('source',oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                whoville.WhoParameter('target',oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString)],
        help="Copy the Field state (defined, active, etc) from one Mesh to another.",
        discussion="""<para>

        This command copies the &field; state from one &mesh; to another,
        meaning that the same &fields; will be defined, active, and
        in-plane in the <varname>target</varname> &mesh; as in the
        <varname>source</varname> &mesh;.  If &fields; were explicitly
        <link linkend='MenuItem:OOF.Mesh.Initialize'>initialized</link> in
        the source &mesh;, the initializers will be copied, but the
        command does <emphasis>not</emphasis> copy the &field; values.
        (This is because the source and target meshes might have quite
        different geometries.)

        </para>"""
                            ) )



    # Likewise for equation state.
    def _copyEquationState(menuitem, source, target):
        if source == target:
            return
        source_spblm = oof2.engine.subproblemcontext.subproblems[source]
        target_spblm = oof2.engine.subproblemcontext.subproblems[target]


        source_spblm.begin_reading()
        target_spblm.reserve()
        target_spblm.begin_writing()
        try:
            source_obj = source_spblm.getObject()
            target_obj = target_spblm.getObject()
            ##
            source_eqns = source_obj.active_equations
            target_eqns = target_obj.active_equations

            source_kin_eqns = source_obj.active_kinetics
            target_kin_eqns = target_obj.active_kinetics

            source_dyn_eqns = source_obj.active_dynamics
            source_dyn_eqns = target_obj.active_dynamics

            for equationname in target_eqns.keys():
                e = target_eqns[equationname]
                if not source_obj.is_active_equation(e):
                    target_obj.deactivateEquation(e)
                    switchboard.notify("equation activated", target, e.name(), 0)
                if not source_obj.is_kinetically_active_equation(e):
                    target_obj.deactivateKinetics(e)
                    switchboard.notify('kinetics activated', target, e.name(), 0)
                if not source_obj.is_dynamically_active_equation(e):
                    target_obj.deactivateDynamics(e)
                    switchboard.notify('dynamics activated', target, e.name(), 0)
            for equationname in source_eqns.keys():
                e = source_eqns[equationname]
                if not target_obj.is_active_equation(e):
                    target_obj.activateEquation(e)
                    switchboard.notify("equation activated", target, e.name(), 1)
                if not target_obj.is_kinetically_active(e):
                    target_obj.activateKinetics(e)
                    switchboard.notify('kinetics activated', target, e.name(), 1)
                if not target_obj.is_dynamically_active(e):
                    target_obj.activateDynamics(e)
                    switchboard.notify('dynamics activated', target, e.name(), 1)
        finally:
                    source_spblm.end_reading()
                    target_spblm.end_writing()
                    target_spblm.cancel_reservation()
                    switchboard.notify("redraw")
        ## oof2.engine.subproblemcontext.subproblems[target].changed()



    eqnmenu.addItem(oofmenu.OOFMenuItem(
        'Copy_Equation_State',
        callback=_copyEquationState,
        threadable=oofmenu.THREADABLE,
        params=[whoville.WhoParameter('source',oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                whoville.WhoParameter('target', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString)],
        help="Copy the set of active Equations from one Mesh to another.",
        discussion="""<para>

        This command copies the &equation; state from one &mesh; to
        another, meaning that the same &equations; will be active in the
        <varname>target</varname> &mesh; as in the
        <varname>source</varname> &mesh;.

        </para>"""

        ) )


    #######################################

    solvermenu = subproblemMenu.addItem(oofmenu.OOFMenuItem(
        'Solver',
        help="Commands for setting solvers on subproblems",
        cli_only=1))

    def _setsolver(menuitem, subproblem, matrixdriver, timedriver):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.setMatrixDriver(matrixdriver)
            prob.setTimeDriver(timedriver)
        finally:
            subproblemcontext.cancel_reservation()
    solvermenu.addItem(oofmenu.OOFMenuItem(
        'SetSolver',
        callback=_setsolver,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem', oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                parameter.RegisteredParameter('matrixdriver', solverdriver.Driver,
                                              tip=parameter.emptyTipString),
                parameter.RegisteredParameter('timedriver', timedrivers.TimeDriver)
                ],
        help="Set mesh driver and time driver of subproblem"
        ))



    def _solve(menuitem, subproblem, matrixdriver, timedriver):
            debug.fmsg('solving')
            solverProgressBar = menuitem.getProgressBar("continuous") ## actually ran by the worker
            subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
            subproblemObject = subproblemcontext.getObject()
            meshcontext = subproblemcontext.getMesh()
            meshcontext.reserve()
            try:
                subproblemObject.initializeMesh(meshcontext) ## selects elements, activates fields and eqns
                timedriver.apply(matrixdriver, meshcontext)
                ## solved flag is set even if solution did NOT
                ## converge in order to show how far the solution
                ## went.
                meshcontext.solved()
                subproblemObject.restoreMesh(meshcontext) ## unselects elements, unactivates fields and eqns
            finally:
                meshcontext.cancel_reservation()

    solvermenu.addItem(oofmenu.OOFMenuItem(
        'Solve',
        callback=_solve,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem',
                                      oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                parameter.RegisteredParameter('matrixdriver', solverdriver.Driver,
                                              tip=parameter.emptyTipString),
                parameter.RegisteredParameter('timedriver', timedrivers.TimeDriver)
                ],
        help="Solve a problem on a specified region of the mesh",
        ## discussion=xmlmenudump.loadFile('')
        ))

    def _reset(menuitem, subproblem):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemObject = subproblemcontext.getObject()
        meshcontext = subproblemcontext.getMesh()
        meshcontext.reserve()
        try:
            femesh = meshcontext.getObject()
            ## first set time indices back to zero
            femesh.set_time(0)
            ## next set active fields back to zero
            resetObj = fieldinit.ResetFieldInit()
            for fld in field.allFields:
                if fld.name() in subproblemObject.active_fields:
                    resetObj.apply(femesh, fld)
            switchboard.notify("field initialized")
            ## then apply any initializers that had been
            ## applied
            initializers_dict = meshcontext.initializers
            for fld in initializers_dict.keys():
                initializer = initializers_dict[fld]
                try:
                    initializer.apply(femesh, fld)
                except:
                    reporter.warning("could not re-apply initializer: ",  initializer)
            ## finally notify that solution is not current and
            ## redraw
            meshcontext.solution_is_current = 0
            meshcontext.changed()
            switchboard.notify("redraw")
        finally:
            meshcontext.cancel_reservation()


    solvermenu.addItem(oofmenu.OOFMenuItem(
        'Reset',
        callback=_reset,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('subproblem',
                                      oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString)
                ],
        help="Solve a problem on a specified region of the mesh",
        ## discussion=xmlmenudump.loadFile('')
        ))

    def _set_active_region(menuitem, subproblem, skeleton, group):
        subproblemcontext = oof2.engine.subproblemcontext.subproblems[subproblem]
        subproblemcontext.reserve()
        try:
            prob = subproblemcontext.getObject()
            prob.setActiveRegion(skeleton, group)
        finally:
            subproblemcontext.cancel_reservation()



    subproblemMenu.addItem(oofmenu.OOFMenuItem(
        "SetRegion",
        cli_only = 1,
        callback = _set_active_region,
        params=[whoville.WhoParameter('subproblem',
                                      oof2.engine.subproblemcontext.subproblems,
                                      tip=parameter.emptyTipString),
                whoville.WhoParameter("skeleton", whoville.getClass('Skeleton'),
                                      tip=parameter.emptyTipString),
                skeletongroupparams.ElementGroupParameter("group", tip="Name of the group.")],
        help="set area (submesh) that will be solved for"
        )
                           )
