// -*- C++ -*-
// $RCSfile: eulerangle.h,v $
// $Revision: 1.8 $
// $Author: redwing $
// $Date: 2005/02/04 21:15:09 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>


#ifndef EULERANGLE_H
#define EULERANGLE_H

class SphereCoord;
class EulerAngle;

#include "engine/matvec.h"
#include <iostream>

class EulerAngle {
private:
  double alpha_, beta_, gamma_;	// Radians!
public:
  EulerAngle()
    : alpha_(0.), beta_(0.), gamma_(0.) {}
  EulerAngle(double a, double b, double g) // Radians!
    : alpha_(a), beta_(b), gamma_(g) {}
  EulerAngle(const SphereCoord&);
  EulerAngle(const EulerAngle&);
  EulerAngle& operator=(const EulerAngle&);
  
  // return individual components
  double alpha() const { return alpha_; }
  double beta() const { return beta_; }
  double gamma() const { return gamma_; }
  
  // convert to a rotation matrix

  // As used by the Cijkl's "transform" method, these matrices
  // yield lab-frame vectors when right-multiplied by crystal-frame vectors.
  MATRIX_D rotation() const;
  operator MATRIX_D() const { return rotation(); }
  
  // set
  void set(double a, double b, double g) {
    alpha_ = a; beta_ = b; gamma_ = g;
    //std::cerr << "alpha=" << alpha_ << " beta=" << beta_ << " gamma=" << gamma_ << std::endl;
  }
  EulerAngle &operator=(double b) {
    set(0.0, b, 0.0);
    return *this;
  }
  // invert
  EulerAngle operator-() const {
    return EulerAngle(-alpha_, -gamma_, -beta_);
  }
  
  friend std::ostream &operator<<(std::ostream &, const EulerAngle&);
  friend std::istream &operator>>(std::istream &, EulerAngle&);
  
  friend int operator==(const EulerAngle &om1, const EulerAngle &om2);
  
};

class SphereCoord {
private:
  double theta_;
  double phi_;
public:
  SphereCoord() : theta_(0), phi_(0) {}
  SphereCoord(double t, double p) : theta_(t), phi_(p) {}
  SphereCoord(const double * const v);	// find orientation of a vector
  SphereCoord(const VECTOR_D &v); // ditto
  MATRIX_D rotation() const;
  operator MATRIX_D() const { return rotation(); }
  EulerAngle operator-() const { return EulerAngle(-phi_, 0, -theta_); }
  friend std::ostream &operator<<(std::ostream&, const SphereCoord&);
  friend class EulerAngle;
};

#endif

