// -*- C++ -*-
// $RCSfile: nodalequation.h,v $
// $Revision: 1.8 $
// $Author: reida $
// $Date: 2005/07/06 17:57:20 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include <iostream>

// NodalEquations represent one component of an equation at a node.
// Each NodalEquation corresponds to a row of the master stiffness
// matrix.  The row can be determined from the index, locally known
// here, and the FEMesh object's global_equation_map array.
// NodalEquations are re-ordered in the master stiffness matrix
// (relative to the order in the FEMesh's list) in order to put each
// equation in the same row as the column of its conjugate degree of
// freedom.  Unlike DOFs, nodal equations do not have values.  They
// contain index data as well state data indicating if the relevant
// equation is dependent or independent.

typedef unsigned char EQNFLAG;
static const EQNFLAG DEPENDENT = 1;
static const EQNFLAG SOLVABLE_SUBMESH = 2;

class NodalEquation {
private:
  NodalEquation(int n) : index_(n), flags(0) {}
  ~NodalEquation() {}
  friend class FEMesh;	// only a  FEMesh can create or destroy a NodalEquation

  int index_;
  // double value_;
  EQNFLAG flags;
public:
  int ndq_index() const { return index_; }

  // A "dependent" nodal equation is an equation whose force (rhs) is
  // unknown, and which can be computed *after* the values of all the
  // degrees of freedom are found.  A dependent equation is not solved
  // during the solution step.
  void dependent() { flags |= DEPENDENT; }
  void independent() { flags &= ~DEPENDENT; }
  bool is_dependent() const { return flags & DEPENDENT; }


  void setsubmesh() { flags |= SOLVABLE_SUBMESH; }
  void unsetsubmesh() { flags &= ~SOLVABLE_SUBMESH; }
  bool issubmesh() const { return flags & SOLVABLE_SUBMESH; }
};

std::ostream &operator<<(std::ostream &os, const NodalEquation &neqn);
